import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Wrapperklasse für ein Dreieck auf der Zeichenfläche.
 * 
 * @author Albert Wiedemann 
 * @version 1.0
 */
public class Dreieck extends Actor
{
    /** x-Position der Spitze. */
    protected int x;
    /** y-Position der Spitze. */
    protected int y;
    /** Breite des umgebenden Rechtecks. */
    protected int breite;
    /** Höhe des umgebenden Rechtecks. */
    protected int höhe;
    /** Farbe des Dreiecks. */
    protected String farbe;
    /** Sichtbarkeit des Dreiecks. */
    protected boolean sichtbar;
    /** Drehwinkel (mathematisch positiver Drehsinn) des Dreiecks in Grad. */
    protected int winkel;
    /** Referenz auf das Delegate-Objekt. */
    protected Zeichenfenster.GrafikSymbol symbol;
    /** Referenz auf die eigene Welt. */
    protected GrafikWelt welt;
    /** Referenz auf eigene Bild. */
    protected GreenfootImage img;

    /**
     * Der Konstruktor erzeugt das Delegate-Objekt
     */
    Dreieck ()
    {
        x = 60;
        y = 10;
        breite = 100;
        höhe = 100;
        farbe = "rot";
        sichtbar = true;
        winkel = 0;
        symbol = null;
        img = new GreenfootImage(breite, höhe);
        img.setColor(new Color(255, 255, 255, 0));
        img.fill();
        img.fillPolygon(new int [] {x, x + breite / 2, x - breite / 2, x}, new int [] {y, y + höhe, y + höhe, y}, 4);
        setImage(img);
        super.setRotation(-winkel);
        welt = GrafikWelt.dieWelt;
        symbol = welt.fenster.SymbolErzeugen(Zeichenfenster.SymbolArt.dreieck);
        symbol.PositionSetzen(x - breite / 2, y);
        symbol.GrößeSetzen(breite, höhe);
        symbol.FarbeSetzen(farbe);
        symbol.SichtbarkeitSetzen(false);
        symbol.WinkelSetzen(winkel);
    }
    
    /**
     * Erzeugt das Delegate-Objekt.
     * @param world die Welt, in der das Objekt angelegt wird.
     */
    @Override protected void addedToWorld​(World world)
    {
        super.addedToWorld​(world);
        symbol.SichtbarkeitSetzen(sichtbar);
    }
    
    /**
     * Setzt die Position der Mitte des Dreiecks.
     * @param x x-Position der Mitte
     * @param y y-Position der Mitte
     */
    @Override public void setLocation(int x, int y)
    {
        this.x = x;
        this.y = y - höhe / 2;
        super.setLocation(x, y);
        if (symbol != null)
        {
            symbol.PositionSetzen(this.x - breite / 2, this.y);
        }
    }
    
    /**
     * Setzt den Drehwinkel des Dreiecks.
     * Die Winkelangabe ist in Grad, positive Werte drehen im Uhrzeigersinn,
     * negative Werte drehen gegen den Uhrzeigersinn (mathematisch negativer Drehsinn).
     * @param winkel der (neue) Drehwinkel des Dreiecks
     */
    @Override public void setRotation(int w)
    {
        winkel = -w;
        super.setRotation(w);
        if (symbol != null)
        {
            symbol.WinkelSetzen(winkel);
        }
    }
    
    /**
     * Setzt die Position (der Spitze) des Dreiecks.
     * @param x x-Position der Spitze
     * @param y y-Position der Spitze
     */
    void PositionSetzen(int x, int y)
    {
        this.x = x;
        this.y = y;
        symbol.PositionSetzen(x - breite / 2, y);
        super.setLocation(x, y + höhe / 2);
    }
    
    /**
     * Verschiebt das Dreieck um die angegebenen Werte.
     * @param deltaX Verschiebung in x-Richtung
     * @param deltaY Verschiebung in y-Richtung
     */
    void Verschieben(int deltaX, int deltaY)
    {
        x += deltaX;
        y += deltaY;
        symbol.PositionSetzen(x - breite / 2, y);
        super.setLocation(x, y + höhe / 2);
    }
    
    /**
     * Dreht das Dreieck
     * @param grad Drehwinkel (mathematisch positiver Drehsinn) im Gradmass
     */
    void Drehen(int grad)
    {
        winkel += grad;
        symbol.WinkelSetzen(winkel);
        super.setRotation(-winkel);
    }
        
    /**
     * Setzt die Größe des Dreiecks.
     * @param breite (neue) Breite
     * @param höhe (neue) Höhe
     */
    void GrößeSetzen (int breite, int höhe)
    {
        this.breite = breite;
        this.höhe = höhe;
        symbol.GrößeSetzen(breite, höhe);
        symbol.PositionSetzen(x - breite / 2, y);
        img = new GreenfootImage(breite, höhe);
        img.setColor(new Color(255, 255, 255, 0));
        img.fill();
        img.fillPolygon(new int [] {x, x + breite / 2, x - breite / 2, x}, new int [] {y, y + höhe, y + höhe, y}, 4);
        setImage(img);
        super.setLocation(x, y + höhe / 2);
    }
    
    /**
     * Setzt die Farbe des Dreiecks.
     * Erlaubte Farben sind:
     * "weiß", "weiss", "rot", "grün", "gruen", "blau", "gelb",
     * "magenta", "cyan", "hellgelb", "hellgrün", "hellgruen",
     * "orange", "braun", "grau", "schwarz"
     * Alle anderen Eingaben werden auf die Farbe schwarz abgebildet.
     * @param farbe (neue) Farbe
     */
    void FarbeSetzen (String farbe)
    {
        this.farbe = farbe;
        symbol.FarbeSetzen(farbe);
    }
        
    /**
     * Setzt den Drehwinkel des Dreiecks.
     * Die Winkelangabe ist in Grad,positive Werte drehen gegen den Uhrzeigersinn,
     * negative Werte drehen im Uhrzeigersinn (mathematisch positiver Drehsinn).
     * @param winkel der (neue) Drehwinkel des Dreiecks
     */
    void WinkelSetzen (int winkel)
    {
        this.winkel = winkel;
        symbol.WinkelSetzen(winkel);
        super.setRotation(-winkel);
    }
    
    /**
     * Schaltet die Sichtbarkeit des Dreiecks ein oder aus.
     * Erlaubte Parameterwerte: true, false
     * @param sichtbar (neue) Sichtbarkeit des Dreiecks
     */
    void SichtbarkeitSetzen (boolean sichtbar)
    {
        this.sichtbar = sichtbar;
        symbol.SichtbarkeitSetzen(sichtbar);
    }
        
    /**
     * Entfernt das Dreieck aus dem Zeichenfenster.
     */
    void Entfernen ()
    {
        symbol.Entfernen();
        welt.removeObject(this);
    }
    
    /**
     * Bringt das Dreieck eine Ebene nach vorn.
     */
    void NachVornBringen ()
    {
        symbol.NachVornBringen();
    }
    
    /**
     * Bringt das Dreieck in die vorderste Ebene.
     */
    void GanzNachVornBringen ()
    {
        symbol.GanzNachVornBringen();
    }
    
    /**
     * Bringt das Dreieck eine Ebene nach hinten.
     */
    void NachHintenBringen ()
    {
        symbol.NachHintenBringen();
    }
    
    /**
     * Bringt das Dreieck in die hinterste Ebene.
     */
    void GanzNachHintenBringen ()
    {
        symbol.GanzNachHintenBringen();
    }
}
