import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Wrapperklasse für einen Kreis auf der Zeichenfläche.
 * 
 * @author Albert Wiedemann 
 * @version 1.0
 */
public class Kreis extends Actor
{
    /** x-Position des Kreismittelpunktes. */
    protected int x;
    /** y-Position des Kreismittelpunktes. */
    protected int y;
    /** Radius des Kreises. */
    protected int radius;
    /** Farbe des Kreises. */
    protected String farbe;
    /** Sichtbarkeit des Kreises. */
    protected boolean sichtbar;
    /** Drehwinkel (mathematisch positiver Drehsinn) des Kreises in Grad. */
    protected int winkel;
    /** Referenz auf das Delegate-Objekt. */
    protected Zeichenfenster.GrafikSymbol symbol;
    /** Referenz auf die eigene Welt. */
    protected GrafikWelt welt;
    /** Referenz auf eigene Bild. */
    protected GreenfootImage img;
 
    /**
     * Der Konstruktor macht die Vorbelegungen.
     */
    public Kreis()
    {
        x = 60;
        y = 60;
        radius = 50;
        farbe = "rot";
        sichtbar = true;
        winkel = 0;
        symbol = null;
        img = new GreenfootImage(radius*2, radius*2);
        img.setColor(new Color(255, 255, 255, 0));
        img.fillOval(0, 0, radius*2, radius*2);
        setImage(img);
        super.setRotation(-winkel);
        welt = GrafikWelt.dieWelt;
        symbol = welt.fenster.SymbolErzeugen(Zeichenfenster.SymbolArt.kreis);
        symbol.PositionSetzen(x - radius, y - radius);
        symbol.GrößeSetzen(radius * 2, radius * 2);
        symbol.FarbeSetzen(farbe);
        symbol.SichtbarkeitSetzen(false);
        symbol.WinkelSetzen(winkel);
    }
    
    /**
     * Erzeugt das Delegate-Objekt.
     * @param world die Welt, in der das Objekt angelegt wird.
     */
    @Override protected void addedToWorld​(World world)
    {
        super.addedToWorld(world);
        symbol.SichtbarkeitSetzen(sichtbar);
    }
    
    /**
     * Setzt die Position (des Mittelpunkts) des Kreises.
     * @param x x-Position des Mittelpunkts
     * @param y y-Position des Mittelpunkts
     */
    @Override public void setLocation(int x, int y)
    {
        this.x = x;
        this.y = y;
        super.setLocation(x, y);
        if (symbol != null)
        {
            symbol.PositionSetzen(this.x - radius, this.y - radius);
        }
    }
    
    /**
     * Setzt den Drehwinkel des Kreises.
     * Die Winkelangabe ist in Grad,positive Werte drehen im Uhrzeigersinn,
     * negative Werte drehen gegen den Uhrzeigersinn (mathematisch negativer Drehsinn).
     * @param winkel der (neue) Drehwinkel des Rechtecks
     */
    @Override public void setRotation(int w)
    {
        winkel = -w;
        super.setRotation(w);
        if (symbol != null)
        {
            symbol.WinkelSetzen(winkel);
        }
    }
   
    /**
     * Setzt die Position (des Mittelpunkts) des Kreises.
     * @param x x-Position des Mittelpunkts
     * @param y y-Position des Mittelpunkts
     */
    void PositionSetzen(int x, int y)
    {
        this.x = x;
        this.y = y;
        symbol.PositionSetzen(x - radius, y - radius);
        super.setLocation(x, y);
    }
    
    /**
     * Verschiebt den Kreis um die angegebenen Werte.
     * @param deltaX Verschiebung in x-Richtung
     * @param deltaY Verschiebung in y-Richtung
     */
    void Verschieben(int deltaX, int deltaY)
    {
        x += deltaX;
        y += deltaY;
        symbol.PositionSetzen(x - radius, y - radius);
        super.setLocation(x, y);
    }
    
    /**
     * Dreht den Kreis
     * @param grad Drehwinkel (mathematisch positiver Drehsinn) im Gradmass
     */
    void Drehen(int grad)
    {
        winkel += grad;
        symbol.WinkelSetzen(winkel);
        super.setRotation(-winkel);
    }
        
    /**
     * Setzt den Radius des Kreises.
     * @param radius (neuer) Radius
     */
    void RadiusSetzen (int radius)
    {
        this.radius = radius;
        symbol.GrößeSetzen(radius * 2, radius * 2);
        symbol.PositionSetzen(x - radius, y - radius);
        img = new GreenfootImage(radius*2, radius*2);
        img.setColor(new Color(255, 255, 255, 0));
        img.fillOval(0, 0, radius*2, radius*2);
        setImage(img);
        super.setLocation(x, y);
    }
    
    /**
     * Setzt die Farbe des Kreises.
     * Erlaubte Farben sind:
     * "weiß", "weiss", "rot", "grün", "gruen", "blau", "gelb",
     * "magenta", "cyan", "hellgelb", "hellgrün", "hellgruen",
     * "orange", "braun", "grau", "schwarz"
     * Alle anderen Eingaben werden auf die Farbe schwarz abgebildet.
     * @param farbe (neue) Farbe
     */
    void FarbeSetzen (String farbe)
    {
        this.farbe = farbe;
        symbol.FarbeSetzen(farbe);
    }
        
    /**
     * Setzt den Drehwinkel des Kreises.
     * Die Winkelangabe ist in Grad,positive Werte drehen gegen den Uhrzeigersinn,
     * negative Werte drehen im Uhrzeigersinn (mathematisch positiver Drehsinn).
     * @param winkel der (neue) Drehwinkel des Kreises
     */
    void WinkelSetzen (int winkel)
    {
        this.winkel = winkel;
        symbol.WinkelSetzen(winkel);
        super.setRotation(-winkel);
    }
    
    /**
     * Schaltet die Sichtbarkeit des Kreises ein oder aus.
     * Erlaubte Parameterwerte: true, false
     * @param sichtbar (neue) Sichtbarkeit des Kreises
     */
    void SichtbarkeitSetzen (boolean sichtbar)
    {
        this.sichtbar = sichtbar;
        symbol.SichtbarkeitSetzen(sichtbar);
    }
        
    /**
     * Entfernt den Kreis aus dem Zeichenfenster.
     */
    void Entfernen ()
    {
        symbol.Entfernen();
        welt.removeObject(this);
    }
    
    /**
     * Bringt den Kreis eine Ebene nach vorn.
     */
    void NachVornBringen ()
    {
        symbol.NachVornBringen();
    }
    
    /**
     * Bringt den Kreis in die vorderste Ebene.
     */
    void GanzNachVornBringen ()
    {
        symbol.GanzNachVornBringen();
    }
    
    /**
     * Bringt den Kreis eine Ebene nach hinten.
     */
    void NachHintenBringen ()
    {
        symbol.NachHintenBringen();
    }
    
    /**
     * Bringt den Kreis in die hinterste Ebene.
     */
    void GanzNachHintenBringen ()
    {
        symbol.GanzNachHintenBringen();
    }
}
